@@ -65,7 +65,6 @@ Cloudscattering results are normalized by :math:`I_0=(\sum b_i)^2` to equal one
 ===============
 
 """
-from __future__ import division
 
 import inspect
 import time
@@ -97,6 +96,16 @@ def _fa_sphere(qr):
     return fa
 
 
+def _fa_coil(qrg):
+    """
+    qrg is array dim 1
+    fa_coil**2 is Debye function see [2]_ in  sphereCoreShellGaussianCorona
+    """
+    fa = np.ones(qrg.shape)
+    fa[qrg != 0] = (1 - np.exp(-qrg[qrg > 0])) / (qrg[qrg > 0])
+    return fa
+
+
 def _scattering(point, r, q, blength, iff, formfactoramp=None, rms=0, ffpolydispersity=0):
     """
     Coherent scattering of objects at positions r in direction point on sphere with length (radius) q
@@ -263,11 +272,16 @@ def cloudScattering(q, cloud, relError=50, formfactor=None, V=None, rms=0, ffpol
          - 'gauss' : Gaussian function  :math:`b_i(q)=b V exp(-\pi V^{2/3}q^2)` according to [2]_
                      equal for all cloud points. The Gaussian shows no artificial minima compared to the sphere.
                      Parameter V needed to determine :math:`b_i`.
-         - Explicit isotropic formfactor as array with [q,ff1(q),ff2(q),ff3(q),....].
-            - If multiple formfactors are given the formfactor for a cloud point needs to be given in cloud[4]
-            - The normalized scattering amplitude fa for each cloud point is calculated as fa=ffi**0.5/fai(0).
+         - 'coil' :  Polymer coil (ideal Gaussian chain) showing scattering according to Debye function.
+                     Parameter V needed to determine :math:`R_g = (\frac{3V}{4\pi})^{1/3}`.
+                     The scattering length is :math:`b_i = Nb_{monomer}` with monomer number :math:`N`
+         - Explicit isotropic :math:`F_a(q)` as array with [q,fa1(q),fa2(q),fa3(q),....].
+            - If multiple fai are given the index i for a cloud point needs to be given in cloud[4]
+            - The normalized scattering amplitude fa for each cloud point is calculated as fa=fai/fai[0].
               Missing values are linear interpolated (np.interp), q values outside interval are mapped to qmin or qmax.
             - Explicit formfactors are assumed to be isotropic.
+            - If the scattering amplitude is not known :math:`F_a(q) \approx F^{1/2}(q)`
+              might be used as approximation for low Q.
     V : float, default=None
         Volume of the scatterers to determine scattering amplitude (see formfactor).
         Only needed for formfactor 'sphere' and 'gauss'.
@@ -287,7 +301,8 @@ def cloudScattering(q, cloud, relError=50, formfactor=None, V=None, rms=0, ffpol
     Returns
     -------
     dataArray
-        Columns [q, Pq, beta]
+        Columns [q, Pq, beta, fa]
+         - Pq , formfactor , beta asymmetry factor, fa scattering amplitude
          - .I0 :          :math:`=I(q=0)=(\sum_N b_i)^2`
          - .sumblength :  :math:`=\sum_N b_i`
          - .formfactoramplitude   : formfactor amplitude of cloudpoints according to type for all q values.
@@ -398,12 +413,15 @@ def cloudScattering(q, cloud, relError=50, formfactor=None, V=None, rms=0, ffpol
     An objekt with **explicit given formfactor** for each gridpoint.
     ::
 
+     import jscatter as js
+     import numpy as np
+     q = js.loglist(0.01, 7, 100)
      # 5 coreshell particles in line with polydispersity
      rod0 = np.zeros([5, 3])
      rod0[:, 1] = np.r_[0, 1, 2, 3, 4] * 4
-     q = js.loglist(0.01, 7, 100)
      cs = js.ff.sphereCoreShell(q=q, Rc=1, Rs=2, bc=0.1, bs=1, solventSLD=0)
-     ffe = js.ff.cloudScattering(q, rod0, formfactor=cs,relError=100,ffpolydispersity=0.1)
+     csa = np.c_[cs.X,cs[2]].T
+     ffe = js.ff.cloudScattering(q, rod0, formfactor=csa,relError=100,ffpolydispersity=0.1)
      p=js.grace()
      p.plot(ffe)
 
@@ -603,9 +621,16 @@ def cloudScattering(q, cloud, relError=50, formfactor=None, V=None, rms=0, ffpol
             formfactoramp = np.c_[q, _fa_sphere(q * R)].T
             formfactor = 'sphere'
             iff = np.ones(cloud.shape[0], dtype=int)
+        elif formfactor.startswith('c'):
+            # polymer coil showing Debye scattering
+            Rg = (3. * V / 4. / np.pi) ** (1 / 3.)
+            formfactoramp = np.c_[q, _fa_coil(q * Rg)].T
+            formfactor = 'polymer'
+            iff = np.ones(cloud.shape[0], dtype=int)
+
     elif isinstance(formfactor, np.ndarray):
         formfactoramp = formfactor.copy()
-        formfactoramp[1:] = formfactor[1:] ** 0.5 / formfactor[1:, :1] ** 0.5
+        formfactoramp[1:] = formfactor[1:] / formfactor[1:, :1]
         formfactor = 'explicit'
     else:
         # const form factor as default
@@ -644,15 +669,13 @@ def cloudScattering(q, cloud, relError=50, formfactor=None, V=None, rms=0, ffpol
                                      ncpu=ncpu, relError=relError, loopover='q', output=False)
 
         res = np.array(res).T
-        # the third row is F(Q) but will be asymmetry factor beta according to Chen
-        #  which is beta=|<F(Q)>|²/<|F(Q)|²>   and scattering amplitude F(Q)
-        res[2] = (res[2] * res[2].conj())
-        if sumblength != 0:
-            # prevent division by zero in some cases (matching)
-            res[2] = res[2] / res[1]  # beta
-            res[1] = res[1] / sumblength ** 2  # formfactor normalisation
-        result = dA(res[:3], dtype=np.float)
-        result.columnname = 'q; Pq; beta'
+        # the third row is F(Q)
+        # asymmetry factor beta according to Chen  beta=|<F(Q)>|²/<|F(Q)|²>
+        beta = (res[2] * res[2].conj()) / res[1]
+        res[1] = res[1] / sumblength ** 2  # normalisation
+        res[2] = res[2] / sumblength       # normalisation
+        result = dA(np.c_[res[0], res[1], beta, res[2]].T, dtype=np.float)
+        result.columnname = 'q; Pq; beta; fa'
 
     result.sumblength = sumblength
     result.I0 = sumblength ** 2
@@ -670,25 +693,33 @@ def cloudScattering(q, cloud, relError=50, formfactor=None, V=None, rms=0, ffpol
 def _coneAverage(qlist, qxzw, qrpt, qfib, r, blength, iff, formfactoramp, rms):
     q, ilist = qlist  # ilist is index in qxzw of the qxzw with same q value
 
-    fa = np.array([np.interp(q, formfactoramp[0, :], amp) for amp in formfactoramp[1:]])
-    fa = blength * fa[iff - 1]
-
-    qfib[:, 0] = q  # cone points around [0,0,1] direction
-    fibpoints = formel.rphitheta2xyz(qfib)  # cartesian coordinates of fibonacci lattice with radius q
     # do the average for all in ilist
     res = {}
     pi_2 = np.pi / 2
-    for il in ilist:
-        # get angles
-        _, p, t = qrpt[il]
-        # rotate fibpoints to angles
-        R = formel.rotationMatrix(formel.rphitheta2xyz(np.r_[1, p - pi_2, pi_2]), t)
-        Rfibpoints = np.einsum('ij,jk->ki', R, fibpoints.T)
-        # take mean of rotated fibpoints
-        if useFortran:
-            res[il] = np.mean([fscatter.cloud.ffx(point, r, fa, rms) for point in Rfibpoints])
-        else:
-            res[il] = np.mean([_sqx(point, r, fa, rms) for point in Rfibpoints])
+    if useFortran:
+        qfib[:, 0] = 1  # cone around [0,0,1] direction; average_ffqxyz needs points on uni sphere
+        fibpoints = formel.rphitheta2xyz(qfib)  # cartesian coordinates of fibonacci lattice with radius q
+        for il in ilist:
+            # get angles
+            _, p, t = qrpt[il]
+            # rotate fibpoints to angles
+            R = formel.rotationMatrix(formel.rphitheta2xyz(np.r_[1, p - pi_2, pi_2]), t)
+            Rfibpoints = np.einsum('ij,jk->ki', R, fibpoints.T)
+            # take mean of rotated fibpoints
+            res[il] = fscatter.cloud.average_ffqxyz(q, r, blength, iff, formfactoramp, rms, 0, Rfibpoints)[1:]
+    else:
+        fa = np.array([np.interp(q, formfactoramp[0, :], amp) for amp in formfactoramp[1:]])
+        fa = blength * fa[iff - 1]
+        qfib[:, 0] = q  # cone points around [0,0,1] direction
+        fibpoints = formel.rphitheta2xyz(qfib)  # cartesian coordinates of fibonacci lattice with radius q
+        for il in ilist:
+            # get angles
+            _, p, t = qrpt[il]
+            # rotate fibpoints to angles
+            R = formel.rotationMatrix(formel.rphitheta2xyz(np.r_[1, p - pi_2, pi_2]), t)
+            Rfibpoints = np.einsum('ij,jk->ki', R, fibpoints.T)
+            # take mean of rotated fibpoints
+            res[il] = np.mean([_sqx(point, r, fa, rms) for point in Rfibpoints], axis=0)
 
     return res
 
@@ -720,7 +751,7 @@ def _sqx(qx, r, fa, rms=0):
     iqr = np.einsum('i,ji', qx, r) * 1j
     beiqrsum = np.einsum('i,i', fa, np.exp(iqr))
     Sq = beiqrsum * beiqrsum.conj()
-    return Sq.real
+    return Sq.real, beiqrsum.real
 
 
 def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None, formfactor=None, ncpu=0):
@@ -760,11 +791,16 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
          - 'gauss' : Gaussian function  :math:`b_i(q)=b V exp(-\pi V^{2/3}q^2)` according to [2]_.
                      The Gaussian shows no artificial minima compared to spheres.
                      The scattering length per particle is :math:`b_i = V* contrast`
-         - Explicit isotropic formfactor as array with [q,ff1(q),ff2(q),ff3(q),....].
-            - If multiple formfactors are given the formfactor for a cloud point needs to be given in cloud[4]
-            - The normalized scattering amplitude fa for each cloud point is calculated as fa=ffi**0.5/fai(0).
+         - 'coil' :  Polymer coil (ideal Gaussian chain) showing scattering according to Debye function.
+                     Parameter V needed to determine :math:`R_g = (\frac{3V}{4\pi})^{1/3}`.
+                     The scattering length is :math:`b_i = Nb_{monomer}` with monomer number :math:`N`
+         - Explicit isotropic :math:`F_a(q)` as array with [q,fa1(q),fa2(q),fa3(q),....].
+            - If multiple fai are given the index i for a cloud point needs to be given in cloud[4]
+            - The normalized scattering amplitude fa for each cloud point is calculated as fa=fai/fai[0].
               Missing values are linear interpolated (np.interp), q values outside interval are mapped to qmin or qmax.
             - Explicit formfactors are assumed to be isotropic.
+            - If the scattering amplitude is not known :math:`F_a(q) \approx F^{1/2}(q)`
+              might be used as approximation for low Q.
     V : float, default=None
         Volume of the scatterers to determine scattering amplitude (see formfactor).
         Only needed for formfactor 'sphere' and 'gauss'.
@@ -833,10 +869,10 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
     Here we add a isotropic Percus-Yevick structure factor as effective composite interaction
     between particle composites. ::
 
-     N=3    # number of particles
-     d=16.0  # distance of particles
+     N=6    # number of particles
      Rc=2   # core radius
-     Rs=8   # outer shell radius
+     Rs=6   # outer shell radius
+     d=Rs*2 # distance of particles
      volumefraction=0.10  # single particles
      rms=1.5
      # position composite particles along X axis
@@ -847,10 +883,11 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
      # core shell formfactor for particles and use interpolation
      q = js.loglist(0.01, 7, 100)
      cs = js.ff.sphereCoreShell(q=q, Rc=Rc, Rs=Rs, bc=-0.1, bs=1, solventSLD=0)
+     csa = np.c_[cs.X,cs[2]].T
      # oriented composite scattering
-     ffe = js.ff.orientedCloudScattering(qxzw, rod0, formfactor=cs, coneangle=5, nCone=100, rms=rms)
+     ffe = js.ff.orientedCloudScattering(qxzw, rod0, formfactor=csa, coneangle=5, nCone=100, rms=rms)
      fig4a = js.mpl.contourImage(ffe.X, ffe.Z, ffe.Y, colorMap='gnuplot',scale='log')
-     fig4a.axes[0].set_title('3 core shell particles with R=2 along Z with noise')
+     fig4a.axes[0].set_title('3 core shell particles with R=2 along X with noise')
 
      # add structure factor according to radial q component
      sf=js.sf.PercusYevick(q, Rs*N/2, eta=volumefraction*N)  # approximate higher radius
@@ -882,10 +919,10 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
 
      # or use interpolation; cs will be our formfactor
      q = js.loglist(0.01, 7, 100)
-     cs = js.ff.sphereCoreShell(q=q, Rc=1, Rs=2, bc=0.1, bs=1, solventSLD=0)
+     csa = js.ff.sphereCoreShell(q=q, Rc=1, Rs=2, bc=0.1, bs=1, solventSLD=0)[[0,2]]
 
      # calc scattering in plane qxzw
-     ffe = js.ff.orientedCloudScattering(qxzw, rod0, formfactor=cs, coneangle=20, nCone=100, rms=0.05)
+     ffe = js.ff.orientedCloudScattering(qxzw, rod0, formfactor=csa, coneangle=20, nCone=100, rms=0.05)
 
      # show it in surface plot
      fig4 = js.mpl.surface(ffe.X, ffe.Z, np.log10(ffe.Y), colorMap='gnuplot')
@@ -918,7 +955,7 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
      for alpha in np.r_[0:91:30]:
          R=js.formel.rotationMatrix(np.r_[0,0,1],np.deg2rad(alpha)) # rotate around Z axis
          qa=np.dot(R,qxzw.T).T[:,:2]
-         ffe = js.ff.orientedCloudScattering(qa, rod0, formfactor=cs, coneangle=20, nCone=100, rms=0.05)
+         ffe = js.ff.orientedCloudScattering(qa, rod0, formfactor=csa, coneangle=20, nCone=100, rms=0.05)
          p.plot(x,ffe.Y,li=[1,2,-1],sy=0,le='alpha=%g' %alpha)
      p.xaxis(label=r'Q / nm\S-1')
      p.legend()
@@ -979,9 +1016,15 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
             formfactoramp = np.c_[qred, _fa_sphere(qred * R)].T
             formfactor = 'sphere'
             iff = np.ones(cloud.shape[0], dtype=int)
+        elif formfactor.startswith('c'):
+            # polymer coil showing Debye scattering
+            Rg = (3. * V / 4. / np.pi) ** (1 / 3.)
+            formfactoramp = np.c_[qred, _fa_coil(qred * Rg)].T
+            formfactor = 'polymer'
+            iff = np.ones(cloud.shape[0], dtype=int)
     elif isinstance(formfactor, np.ndarray):
         formfactoramp = formfactor.copy()
-        formfactoramp[1:] = formfactor[1:] ** 0.5 / formfactor[1:, :1] ** 0.5
+        formfactoramp[1:] = formfactor[1:] / formfactor[1:, :1]
         formfactor = 'explicit'
     else:
         # const form factor as default
@@ -994,9 +1037,8 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
                              qxzw=qxzw, qrpt=qrpt, qfib=qfib, r=cloud, blength=blength, iff=iff,
                              formfactoramp=formfactoramp, rms=rms,
                              ncpu=ncpu, loopover='qlist', output=False)
-
     # distribute Sxy according to saved indices from qlist
-    Sq = np.zeros_like(qxzw[:, 0])
+    Sq = np.zeros((qxzw.shape[0], 2))
     for res1 in res:
         for ii, Sxy in res1.items():
             Sq[ii] = Sxy
@@ -1008,7 +1050,7 @@ def orientedCloudScattering(qxzw, cloud, rms=0, coneangle=10, nCone=50, V=None,
     if isinstance(V, numbers.Number):
         result.Volume = V
     result.setColumnIndex(ix=0, iy=3, iz=1, iw=2, iey=None)
-    result.columnname = 'qx; qz; qw; Sq'
+    result.columnname = 'qx; qz; qw; Sq; fa'
 
     result.modelname = inspect.currentframe().f_code.co_name
     return result
