@@ -21,7 +21,7 @@
 """
 Functions to read DLS data, apply the CONTIN analysis and plot the CONTIN results.
 
-This is a wrapper around the original Fortran code of S. Provencher.
+This is a wrapper around the original Fortran code of S. Provencher [1,2,3]_ .
 As the license is unclear please download and install if needed.
 
 For CONTIN refer to  http://s-provencher.com/contin.shtml
@@ -47,23 +47,23 @@ For CONTIN refer to  http://s-provencher.com/contin.shtml
 
 References
 
-.. [1]  S.W. Provencher:
-        Inverse problems in polymer characterization: Direct analysis of polydispersity with photon correlation
-        spectroscopy.
-        Makromol. Chem. 180, 201 (1979).
-.. [2]  S.W. Provencher:
-        A constrained regularization method for inverting data represented by linear algebraic or integral equations.
-        Comput. Phys. Commun. 27, 213 (1982).
-.. [3]  S.W. Provencher:
-        CONTIN: A general purpose constrained regularization program for inverting noisy linear algebraic and integral
-        equations.
-        Comput. Phys. Commun. 27, 229 (1982).
+ S.W. Provencher:
+ Inverse problems in polymer characterization: Direct analysis of polydispersity with photon correlation
+ spectroscopy.
+ Makromol. Chem. 180, 201 (1979).
+
+ S.W. Provencher:
+ A constrained regularization method for inverting data represented by linear algebraic or integral equations.
+ Comput. Phys. Commun. 27, 213 (1982).
+
+ S.W. Provencher:
+ CONTIN: A general purpose constrained regularization program for inverting
+ noisy linear algebraic and integral equations.
+ Comput. Phys. Commun. 27, 229 (1982).
 
 
 ---------------
 """
-from __future__ import print_function
-from __future__ import division
 
 import io
 import codecs
@@ -108,6 +108,14 @@ except:
     continexe = ''
 
 
+def _getfirst(a):
+    """ this allows to have float or list of float (take first)"""
+    try:
+        return a[0]
+    except:
+        return a
+
+
 def w2f(word):
     """converts if possible to float"""
     try:
@@ -132,7 +140,7 @@ def _visc(T, **kw):
 
 
 def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG=-1, timescale=1e-6, **kw):
-    """
+    r"""
     Inverse Laplace transform known as CONTIN analysis developed by Steven Provencher [1,2]_ .
     
     This is a wrapper for the original FORTRAN program from Steven Provencher.
@@ -145,134 +153,133 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
     Parameters
     ----------
     datalist : dataList or single dataArray
-        Correlation data as measured e.g. by DLS
-        Check type parameter dependent on input data g1 or g2!!!!
+        Correlation data as measured e.g. by DLS.
+        Check type parameter dependent on input data g1 or g2!
         Each dataArray is processed.
-    timescale : float; default 1e-6
+    timescale : float, default 1e-6
         Timescale factor. CONTIN uses internal units of seconds.
         As typical instruments use microseconds the time axis needs to be scaled by this factor.
-    Ngrid : int
-        Number of points between min and max.
     tmin,tmax : float,int
-        First/last time value to use in fit
-        If tmin/tmax are negative the number of points are dropped at begin/end.
-        eg min=-5 drops the 5 shortest time points; for max the 5 last points
+        First/last time value to use in fit.
+        If tmin/tmax are negative the number of points are dropped at begin/end of input data.
+        e.g. min=-5 drops the 5 shortest time points; for max the 5 last points
+    Ngrid : int
+        Number of grid points between gmin and gmax.
     gmin/gmax : float
-        First/last time value in distribution
-        If not given calculated from tmin/tmax according to used kernel
+        First/last time value in distribution X scale.
+        If not given calculated from tmin/tmax according to used kernel.
     qgmin/qgmax : float
-        Only for 'relax'.  Same as gmin,gmax except that gmin=qgmin/q^2
+        Only for 'relax'.  Same as gmin,gmax except that gmin=qgmin/q^2.
     qtmin/qtmax : float
-        q^2*t scaled times
+        q^2*t scaled times.
     bgr : float; default 0
-        if >0:assume background and fit it
+        If >0: assume background and fit it.
     typ :  int;  default 0
-         |  0    input is g1 (Field correlation)
-         |        -> means dont change input  in datalist
-         |  1    input is intensity correlation g2 as measured by DLS;
-         |        =>  field correlation is calculated as  g1= sign(g2/R21-1)*abs(g2/R21-1)^0.5
-         | -1    input is g2-1 of DLS,  takes only the square root; sign preserved as for 1
-         |                ALV and Malvern Zetasizer save g2-1
-    distribution : string, default='x',
-        | selects Iuser(10) as {'m':1,'D':2,'L':2,'r':3,'u':4,'x':4,'d':4,'T':4} according to first character of
-        | distribution and RUSER(15,16,17,19). The kernel  g1=l^R23*exp(-R21*t*l^R22) describes
-        | always the field correlation.
-        | 'molweight'  weight fraction molecular weight (mw) distribution l=mw -> mw*exp(-q**2*R18**t*mw**R22)
-        | or 'mass'       R18 to be set in input: relates diffusion coefficient to mol weight
-        |                 R22 to be set in input: exponent eg for polymers IN THETA solvent Diffusion=R18*l^0.5
-        |                 R21 = R20**2*R18=q**2*R18
-        |                 R23=1
-        |                 l=mw and field(0)~ mw for weight fraction (intensity ~ mw**2)
-        | 'Diff'       intensity weight diffusion coefficient distribution with l=Diff -> exp(-q**2*t*Diff);
-        |                 R23 =0
-        |                 R22 =1
-        |                 R21 =R20^2=q^2
-        |                 l is therefore Diffusion coefficient
-        | 'Laplace'    Laplace transform  with exp(-t*Gamma)
-        |                 R23=0
-        |                 R22=1
-        |                 sets R16=0 (wavelength) => R20=undefined and R21=1
-        |                 l is therefore relaxation rate with transform exp(-t*l)
-        | 'radius'     weight fraction radius distribution of spheres satisfying stokes law
-        |                 with l=rh -> rh**3*exp(-q**2* [k_B*T*/(0.06*pi*visc*rh)] * t)
-        |                 R23=3
-        |                 R22=-1
-        |                 R21=k_B*R18*R20^2/(0.06pi*R19)
-        |                 and l=rh is a the hydrodynamic radius
-        |                 weight fraction as field(0) ~ V ~ rh**3 for spheres
-        | 'x' or      intensity weight relaxation times distribution l=T -> exp(-t/T)
-        |                 R21=1, R22=-1, R23=0
-        | 'T1'        intensity weight relaxation times distribution l=T1 -> T1*exp(-t/T1)
-        |                 for nmr rotational diffusion T1 with R23=-1 R21=1, R22=-1 neglecting T2
-        |                 intensity ~ number of contributing atoms ~M ~ V ~ Rh**3
-        |                 rot correlation time tc=(4pi*visc/kT)*Rh**3 -> intensity(0) ~ tc -> R23=1
-        | 'dls'       intensity weight radius distribution of spheres satisfying stokes law
-        |                 with l=rh -> exp(-q**2* [k_B*T*/(0.06*pi*visc*rh)] * t)
-        |                 R21 = k_B*T*/(0.06*pi*visc)
-        |                 R22=-1, R23=0
-        | 'user'       general case RUSER(15,16,17,19,21,22,23) all user specified
-        |                 eg R16=0,R23=0,R22=-1,R21=0 is laplace with Gamma^-1
+         - 0:  input is g1 (field correlation)
+         - 1:  input is intensity correlation g2 as measured by DLS; g1= sign(g2/R21-1)*abs(g2/R21-1)^0.5
+         - -1: input is g2-1, takes only the square root; sign preserved as for 1.
+               ALV and Malvern Zetasizer save g2-1
+    distribution : string, default='x' for DLS,
+        Selects distribution type (Iuser(10)) as {'m':1,'D':2,'L':2,'r':3,'u':4,'x':4,'d':4,'T':4}
+        using RUSER(15,16,17,19).
+
+        The kernel  :math:`g_1=l^{R23}*exp(-R21*t*l^{R22})` describes always the field correlation.
+         - 'x' intensity weight relaxation times distribution l=T -> exp(-t/T)
+               R21=1, R22=-1, R23=0
+         - 'molweight'='mass'  weight fraction molecular weight (mw) distribution l=mw -> mw*exp(-q**2*R18**t*mw**R22)
+            - R18 to be set in input: relates diffusion coefficient to mol weight
+            - R22 to be set in input: exponent eg for polymers IN THETA solvent Diffusion=R18*l^0.5
+            - R21 = R20**2*R18=q**2*R18
+            - R23=1
+            - l=mw and field(0)~ mw for weight fraction (intensity ~ mw**2)
+         - 'Diff' intensity weight diffusion coefficient distribution with l=Diff -> exp(-q**2*t*Diff);
+            - R23 =0
+            - R22 =1
+            - R21 =R20^2=q^2
+            - l is therefore Diffusion coefficient
+         - 'Laplace'    Laplace transform  with exp(-t*Gamma)
+            - R23=0
+            - R22=1
+            - sets R16=0 (wavelength) => R20=undefined and R21=1
+            - l is therefore relaxation rate with transform exp(-t*l)
+         - 'radius'  weight fraction radius distribution of spheres satisfying stokes law with
+            - l=rh -> rh**3*exp(-q**2* [k_B*T*/(0.06*pi*visc*rh)] * t)
+            - R23=3
+            - R22=-1
+            - R21=k_B*R18*R20^2/(0.06pi*R19)
+            - and l=rh is a the hydrodynamic radius
+            - weight fraction as field(0) ~ V ~ rh**3 for spheres
+         - 'T1' intensity weight relaxation times distribution l=T1 -> T1*exp(-t/T1)
+            -   for nmr rotational diffusion T1 with R23=-1 R21=1, R22=-1 neglecting T2
+            -   intensity ~ number of contributing atoms ~M ~ V ~ Rh**3
+            -   rot correlation time tc=(4pi*visc/kT)*Rh**3 -> intensity(0) ~ tc -> R23=1
+         - 'dls' intensity weight radius distribution of spheres satisfying stokes law
+            -    with l=rh -> exp(-q**2* [k_B*T*/(0.06*pi*visc*rh)] * t)
+            - R21 = k_B*T*/(0.06*pi*visc)
+            - R22=-1, R23=0
+         - 'user'       general case RUSER(15,16,17,19,21,22,23) all user specified
+                         eg R16=0,R23=0,R22=-1,R21=0 is laplace with Gamma^-1
     RDG :  1,-1, default = -1   
-        | 1  use rayleighDebyeGans formfactor with wall thickness WALL [cm] of hollow sphere (default WALL =0)
-        | distribution needs to be the 'radius' distribution
+        If 1 use Rayleigh Debye Gans formfactor with wall thickness WALL [cm] of hollow sphere (default WALL =0)
+        distribution needs to be the 'radius' distribution.
     l : float , R16 overrides value in dataArray if given
-        | overrides wavelength in data and set it to 632 nm
-        | (except for Laplace and relaxation then l=0 overrides this because no wavelength is used)
+        Overrides wavelength in data and set it to 632 nm
+        (except for Laplace and relaxation then l=0 overrides this because no wavelength is used).
     n : float, R15, overrides value in dataArray if given
-        refractive index of solvent, default 1.333
+        Refractive index of solvent, default 1.333.
     a : float, R17, overrides value in dataArray if given
-        scattering angle in degrees , default 90°
+        Scattering angle in degrees , default 90°.
     T : float, R18, overrides value in dataArray if given
-        absolute Temperature in K or factor ; default is 293K
+        Absolute Temperature in K or factor ; default is 293K
     WALL : float, R24, overrides value in dataArray if given
-        | wall thickness  in cm for simulation of hollow spheres =0 normal sphere as default
-        | if WALL<=0 will cause the hard sphere to be calculated wall =0
+        Wall thickness  in cm for simulation of hollow spheres.
+         - =0 normal sphere as default
+         - If WALL<0 -> WALL=0
     v : float, R19, overrides value in dataArray if given
-        | viscosity in centipoise; 1cPoise=1e-3 Pa*s
-        | if v='d2o' or 'h2o' the viscosity of d2o or h2o at the given Temperature T is used
-        | default is h2o at T
+        Viscosity in centipoise; 1cPoise=1e-3 Pa*s
+         - If v='d2o' or 'h2o' the viscosity of d2o or h2o at the given Temperature T is used
+         - default is h2o at T
     write : any 
-        if write is given as keyword argument (write=1) the contin output is writen to a file with the '.con' as ending
+        If write is given as keyword argument (write=1) the contin output is writen to a file with '.con' appended.
 
     Returns
     -------
-        | input dataList with best solution added as parameters with name 'contin_'::
-        |     .contin_bestFit               best fit result see below!!!!!!!!!!!!!!!!!
-        |     .contin_result_fit            fitted correlation function
-        |     .contin_fits                  sequence of fits in CONTIN same structure as best Fit
-        |     .contin_alpha                 ALPHA of best solution
-        |     .contin_alphalist             list of ALPHA in fits
-        |     
-        | dataArray with best solution distribution
-        |     .contin_bestFit               [relaxation times,intensity weight ci, errors, hydrodynamic radii,
-        |                                    mass weight ci , number weight ci]
-        |     .contin_bestFit.attr()        shows all available parameters from CONTIN fit and other parameters
-        |     .contin_bestFit.fitquality    measure for the fit quality ALPHA and ...
-        |     .contin_bestFit.peaks         original CONTIN results
-        |                                   as intensity weight relaxation time distribution
-        |     .contin_bestFit.ipeaks        data extracted from the peaks as list for all peaks in a solution
-        |                                   it is a intensity weight relaxation time distribution
-        |                                     [weight                 peak weight
-        |                                       mean                  peak mean
-        |                                       std                   peak standard deviation
-        |                                       mean_err              error of mean
-        |                                       imean                 index of peak mean
-        |                                       1/(q**2*mean)         diffusion constant in cm**2/s
-        |                                       Rh                    hydrodynamic radius in cm
-        |                                       q                     wavevector in 1/cm
-        |                                       1/(q**2*mean)         diffusion constant in nm**2/ns
-        |                                       Rh                    hydrodynamic radius in nm
-        |                                       q                     wavevector in 1/nm
-        |                                       ]
-        |     .contin_bestFit.mpeaks          same  for 2 strongest peaks mass  weight
-        |     .contin_bestFit.npeaks          same  for 2 strongest peaks number weight
-        |     .contin_bestFit.ipeaks_name'    content of ipeaks
-        |     .contin_bestFit.info'
-        |     .contin_bestFit.baseline'       baseline + error
-        |     .contin_bestFit.momentEntireSolution'   all peaks together
-        |     .contin_bestFit.maximum'        maximum value
-        |     .contin_bestFit.filename'  
-        |     .contin_bestFit.imaximum'       index maximum value
+        dataList : input with best solution added as attributes prepended by 'contin_'
+         - .contin_bestFit               **best fit result**
+         - .contin_result_fit            fitted correlation function
+         - .contin_fits                  sequence of "not best fits" in CONTIN with same structure as best Fit
+         - .contin_alpha                 ALPHA of best solution
+         - .contin_alphalist             list of ALPHA in fits
+
+        bestFit dataArray with best solution distribution content
+         - .contin_bestFit contains  [relaxation times,intensity weight ci, errors, hydrodynamic radii,
+                                       mass weight ci , number weight ci]
+         - .contin_bestFit.attr()      shows all available parameters from CONTIN fit and other parameters
+         - .contin_bestFit.fitquality  measure for the fit quality ALPHA and ...
+         - .contin_bestFit.peaks       original CONTIN results as intensity weight relaxation time distribution
+         - .contin_bestFit.ipeaks      data extracted from the peaks as list for all peaks in a solution
+           it is a intensity weight relaxation time distribution with content
+            - [weight                 peak weight
+            -   mean                  peak mean
+            -   std                   peak standard deviation
+            -   mean_err              error of mean
+            -   imean                 index of peak mean
+            -   1/(q**2*mean)         diffusion constant in cm**2/s
+            -   Rh                    hydrodynamic radius in cm
+            -   q                     wavevector in 1/cm
+            -   1/(q**2*mean)         diffusion constant in nm**2/ns
+            -   Rh                    hydrodynamic radius in nm
+            -   q]                    wavevector in 1/nm
+
+         - .contin_bestFit.mpeaks      same  for 2 strongest peaks mass  weight
+         - .contin_bestFit.npeaks      same  for 2 strongest peaks number weight
+         - .contin_bestFit.ipeaks_name content of ipeaks
+         - .contin_bestFit.info'
+         - .contin_bestFit.baseline'   baseline + error
+         - .contin_bestFit.momentEntireSolution'   all peaks together
+         - .contin_bestFit.maximum'    maximum value
+         - .contin_bestFit.filename'
+         - .contin_bestFit.imaximum'   index maximum value
      
     Notes
     -----
@@ -296,7 +303,9 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
      fp1=np.trapz(ff.Y[p1],ff.X[p1])/np.trapz(ff.Y,ff.X) # fraction of full signal
 
     **An example dataset with noise**
-    Please test noise
+    In this synthetic example the noise causes a peak width of about 22% (std/mean) as it is
+    typically found for monodisperse samples (e.g. proteins).
+    This contribution is not caused by polydispersity and always present dependent on noise level.
     ::
 
      import jscatter as js
@@ -306,26 +315,35 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
      D=0.05*1000  # nm**2/ns * 1000 = units nm**2/microseconds
      noise=0.0001  # typical < 1e-3
      data=js.dA(np.c_[t,0.95*np.exp(-q**2*D*t)+noise*np.random.randn(len(t))].T)
-     #Add attributes to overwrite defaults
+     # add attributes to overwrite defaults
      data.Angle      =90    # scattering angle in degrees
      data.Temperature=293   # Temperature of measurement  in K
      data.Viscosity  =1     # viscosity cPoise
      data.Refractive =1.333 # refractive index
      data.Wavelength =632   # wavelength
-     p=js.grace()
-     p.plot(data,legend='simulated data')
-     p.xaxis(scale='l')
      # do CONTIN
      dr=js.dls.contin(data,distribution='x')
      js.dls.contin_display(dr)         # display overview
      # same but do it yourself
-     p.plot(dr,li=1,sy=0,le='contin')
-     p.plot(dr.contin_bestFit[0].X,dr.contin_bestFit[0].Y*100,sy=2)
+     # plot results
+     p=js.grace(1.4,1)
+     p.plot(data,sy=[1,1,1],legend='simulated data')
+     p.xaxis(scale='log',label=r'\xG\f{} / µs',min=1,max=1e5)
+     p.yaxis(label=r'G\s1\N / P(\xG\f{})')
+     p.plot(dr,li=[1,3,3],sy=0,le='contin')
+     p.plot(dr.contin_bestFit[0].X,dr.contin_bestFit[0].Y*100,sy=2,le='distribution')
      p.legend()
+     # p.save(js.examples.imagepath+'/contin.jpg')
      # access peak values by
      dr[0].contin_bestFit.ipeaks_name
      dr[0].contin_bestFit.ipeaks
+     # calc std/mean of peak
+     dr[0].contin_bestFit.ipeaks[0,2]/dr[0].contin_bestFit.ipeaks[0,1]
 
+    .. image:: ../../examples/images/contin.jpg
+     :align: center
+     :width: 50 %
+     :alt: sphere
 
     R20 (scattering vector) is calculated as R20= 4e-7*pi*R15/R16*sin(R17/2), if R16!=0. else R20=0
 
@@ -433,7 +451,7 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
                         'Please compile and place executable at callable path. ' +
                         'See documentation of DLS module.')
 
-    if datalist._isdataArray:
+    if hasattr(datalist, '_isdataArray'):
         datalist = dL(datalist)
     elif isinstance(datalist, str):
         datalist = dL(datalist)
@@ -620,56 +638,47 @@ def contin(datalist, Ngrid=256, tmin=-2, tmax=-2, bgr=0, distribution='dls', RDG
     print('processing  %i datasets ' % len(datalist))
     for data in datalist:
         idata += 1
-        print(idata)
+        print('evaluate Nr.:', idata)
         try:
-            file = getattr(data, '@name')[0]
+            file = getattr(data, '@name')
         except AttributeError:
             file = time.strftime("%y%m%d%H%M%S", time.localtime())
         header[0] = file
         data.extract_comm(deletechars=':[]()"')  # find extra parameters in comments
-        # data[0]*=0.001                                       # have all in s not in ms from ALV
+
         # extract datetime from comments from ALV instrument
-        timestr = ''
-        for line in data.comment:
-            if line[:4] == 'Time' or line[:4] == 'Date':
-                try:
-                    timestr += line.translate(None, '"').split()[2]
-                    datetime = time.mktime(time.strptime(timestr, "%d.%m.%Y%H:%M:%S"))
-                except:
-                    pass
-        # noinspection PyBroadException
         try:
-            data.datetime = datetime
+            timestr = list(filter(lambda ll: ll.startswith('Time') or ll.startswith('Date'), data.comment))[0]
+            timestr = timestr.translate(None, '"').split()[2]
+            data.datetime = time.mktime(time.strptime(timestr, "%d.%m.%Y%H:%M:%S"))
         except:
-            data.datetime = 0
+            data.datetime=0
         # take values from data
-        # this allows to have float or list of float (take first)
-        _getfirst = lambda a: a if isinstance(data.Angle, numbers.Number) else a[0]
         if 'n' not in kw:
             try:
                 n = _getfirst(data.Refractive)
-            except:
+            except AttributeError:
                 pass
         if 'l' not in kw and distribution[0] != 'L' and 'R16' not in kw:
             # noinspection PyBroadException
             try:
                 l = _getfirst(data.Wavelength)
-            except:
+            except AttributeError:
                 pass
         if 'a' not in kw:
             try:
                 a = _getfirst(data.Angle)
-            except:
+            except AttributeError:
                 pass
         if 'T' not in kw:
             try:
                 T = _getfirst(data.Temperature)
-            except:
+            except AttributeError:
                 pass
         if 'v' not in kw:
             try:
                 v = _getfirst(data.Viscosity)
-            except:
+            except AttributeError:
                 v = _visc(T, **kw)
         else:
             v = _visc(T, **kw)
