@@ -17,10 +17,8 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-from __future__ import division
-from __future__ import print_function
 
 # Files in this folder (libs) may be under a different copyright than Jscatter as they
-# are original from different sources under a Open Source License.
+# are original from different sources under a Open Source License or without explicit license.
 
-# Please check the files for the respective License.
+# Please check the files for the respective license.
