@@ -66,6 +66,7 @@ asciitext = """# these are really ugly data
  # temp     ;    1000 1 2 3 4 5 6
  # pressure ; 1013 12  bar
  # @name     ; temp2bsa
+ FreQuency MHz 3.141 bla 
  &doit
  link @linktest
  1 2 3 0.
@@ -116,11 +117,14 @@ class dataArrayTest(unittest.TestCase):
                            ignore='')
         self.data5 = js.dA('asciitestfile.dat', replace={'#': '', ';': '', ',': '.'},
                            skiplines=lambda w: any(a in ['**', 'nan'] for a in w), ignore='', index=1)
+        self.data5.getfromcomment('frequency', convert=lambda a: float(a.split()[1]), ignorecase=True)
         self.data5.save('asciitestfile2.dat')
         self.data6 = js.dA('asciitestfile2.dat')
 
         self.data7 = js.dA(np.c_[x, x, x * 0 + 0.001].T)
 
+        self.data8 = js.dA(os.path.join(js.examples.datapath, 'iqt_1hho.dat'),index=5)
+
         self.setupdone = True
 
     @classmethod
@@ -149,9 +153,21 @@ class dataArrayTest(unittest.TestCase):
         nptest.assert_array_almost_equal(interpolateALL.Y, [1, 3., 4.5, 3.])
         nptest.assert_array_almost_equal(interpolateALL[-1], [1, 3., 4.5, 3.])
 
-    def test_stuff(self):
+    def test_attributtes(self):
         # test if this results in single float/array
         self.assertEqual(self.data.max(), 9.9)
+        self.assertEqual(self.data8.Drot, 1.618e-05)
+        # test columnnames
+        self.assertEqual(self.data8.columnname, 'time; Sqt; Sqt0')
+        # equal return and type from _ and ['string']
+        nptest.assert_array_almost_equal(self.data8._time, self.data8['time'])
+        self.assertEqual(type(self.data8._time), type(self.data8['time']))
+        self.assertEqual(type(self.data8[0]), js.dataarray.dataArray)
+        # raise error for ndim<2
+        with self.assertRaises(IndexError):
+            self.data8[0]._time
+        with self.assertRaises(IndexError):
+            self.data8[0]['time']
 
     def test_polyfit(self):
         t = np.r_[0:100:]
@@ -200,6 +216,7 @@ class dataArrayTest(unittest.TestCase):
         self.assertEqual(self.data4[2, 4], 3)
         self.assertEqual(self.data4.pressure[-1], 'bar')
         self.assertEqual(self.data5.link.X[1], 2.0)
+        self.assertEqual(self.data5.frequency, 3.141)
         self.assertEqual(self.data6.link.X[1], self.data5.link.X[1])
         self.assertEqual(self.data6.comment[1], self.data5.comment[1])
 
@@ -444,10 +461,38 @@ class formelTest(unittest.TestCase):
         diff_g = js.formel.parQuadratureSimpson(js.dynamic.simpleDiffusion, -3 * ds + D, 3 * ds + D, parname='D',
                                                 weights=pdf, tol=0.01, q=0.5, t=t)
         gaussint = js.formel.parQuadratureAdaptiveGauss(js.formel.gauss, -20, 120, 'x', mean=50, sigma=10)
-
         self.assertTrue(all(np.abs((diff.Y - diff_g.Y)) < 0.005))
         self.assertAlmostEqual(gaussint.Y[0], 1, 7)
 
+        q = np.r_[0.1:5.1:0.1]
+        def gauss(x, mean, sigma):
+            return np.exp(-0.5 * (x - mean) ** 2 / sigma ** 2) / sigma / np.sqrt(2 * np.pi)
+        # 1dimensional
+        def gauss1(q, x, mean=0, sigma=1):
+            g = np.exp(-0.5 * (x[:, None] - mean) ** 2 / sigma ** 2) / sigma / np.sqrt(2 * np.pi)
+            return g * q
+        # 3 dimensional
+        def gauss3(q, x, y=0, z=0, mean=0, sigma=1):
+            g = gauss(x, mean, sigma) * gauss(y, mean, sigma) * gauss(z, mean, sigma)
+            return g[:, None] * q
+        pQFGxD_g1 = js.formel.pQFGxD(gauss1, 0, 100, parnames='x', mean=50, sigma=10, q=q, n=15)
+        pQFGxD_g3 = js.formel.pQFGxD(gauss3, [0, 0, 0], [100, 100, 100],
+                                     parnames=['x', 'y', 'z'], mean=50, sigma=10, q=q, n=15)
+        nptest.assert_array_almost_equal(pQFGxD_g1, q, 5)
+        nptest.assert_array_almost_equal(pQFGxD_g3, q, 5)
+
+        # cubature on complex sphere formfactor
+        R = 5
+        pn = ['r', 'theta', 'phi']
+
+        def sphere_complex(r, theta, phi, b, q):
+            fac = b * np.exp(1j * q[:, None] * r * np.cos(theta)) * r ** 2 * np.sin(theta)
+            return fac.T
+        fa_c, err = js.formel.pAC(sphere_complex, [0, 0, 0], [R, np.pi, np.pi * 2], pn, b=1, q=q)
+        # compare to analytic solution
+        sp = js.ff.sphere(q, R)
+        nptest.assert_array_almost_equal(np.real(fa_c*np.conj(fa_c)), sp.Y, 8)
+
         # convolution
         s1 = 3
         s2 = 4
@@ -612,7 +657,7 @@ class sasTest(unittest.TestCase):
                 self.sic = js.sf.latticeFromCIF(siliconcarbide)
                 agbe = js.sf.latticeFromCIF(js.examples.datapath + '/1507774.cif', size=[1, 1, 1])
                 self.lsf = js.sf.latticeStructureFactor(np.r_[0.9:1.3:0.01], lattice=agbe, domainsize=50,
-                                                        rmsd=0.003, wavelength=self.calibration.wavelength[0] / 10)
+                                                        rmsd=0.003, wavelength=0.13414)
 
         except ImportError:
             self.sic = None
@@ -715,8 +760,8 @@ class sasTest(unittest.TestCase):
                                                  [1.31752737, 2.63522907, 3.95327939, 5.27185318, 6.59112589],
                                                  decimal=3)
             # AgBe max around 4.6 at 1.07
-            np.testing.assert_array_almost_equal(self.lsf.Y[16:19],
-                                                 [4.47650566, 4.60622212, 4.558785], decimal=2)
+            np.testing.assert_array_almost_equal(self.lsf[:3, self.lsf.Y.argmax()],
+                                                 [1.07, 4.60, 0.999], decimal=2)
 
 
 # noinspection PyMethodParameters
@@ -861,7 +906,7 @@ def doTest2(verbosity=1):
     """Do only one test."""
     loader = unittest.TestLoader()
     suite = unittest.TestSuite()
-    suite.addTest(loader.loadTestsFromTestCase(dataArrayTest))
+    suite.addTest(loader.loadTestsFromTestCase(sasTest))
     runner = unittest.TextTestRunner(verbosity=verbosity)
     runner.run(suite)
 
